#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

try:
    dim = 10
    R = ot.CorrelationMatrix(dim)
    for i in range(dim):
        for j in range(i):
            R[i, j] = (i + j + 1.0) / (2.0 * dim)
    mean = [2.0] * dim
    sigma = [3.0] * dim
    distribution = ot.Normal(mean, sigma, R)

    size = 100
    sample = distribution.getSample(size)
    sampleY = sample.getMarginal(0)

    sampleZ = ot.Sample(size, 1)
    for i in range(size):
        sampleZ[i, 0] = sampleY[i, 0] * sampleY[i, 0]

    discreteSample1 = ot.Poisson(0.1).getSample(size)
    discreteSample2 = ot.Geometric(0.4).getSample(size)

    # ChiSquared Independance test : test if two samples (of sizes not necessarily equal) are independant ?
    # Care : discrete samples only
    # H0 = independent samples
    # p-value threshold : probability of the H0 reject zone : 0.10
    # p-value : probability (test variable decision > test variable decision evaluated on the samples)
    # Test = True <=> p-value > p-value threshold
    print("ChiSquared=", ot.HypothesisTest.ChiSquared(
        discreteSample1, discreteSample2, 0.10))
    print("ChiSquared2=", ot.HypothesisTest.ChiSquared(
        discreteSample1, discreteSample1, 0.10))

    # Pearson Test : test if two gaussian samples are independent (based on the evaluation of the linear correlation coefficient)
    # H0 : independent samples (linear correlation coefficient = 0)
    # Test = True <=> independent samples (linear correlation coefficient = 0)
    # p-value threshold : probability of the H0 reject zone : 0.10
    # p-value : probability (test variable decision > test variable decision evaluated on the samples)
    # Test = True <=> p-value > p-value threshold
    print("Pearson=", ot.HypothesisTest.Pearson(sampleY, sampleZ, 0.10))

    ot.RandomGenerator.SetSeed(0)
    sample1 = ot.Normal().getSample(20)
    sample2 = ot.Normal(0.1, 1.1).getSample(30)
    resultTwoSamplesKolmogorov = ot.HypothesisTest.TwoSamplesKolmogorov(
        sample1, sample2)
    print(resultTwoSamplesKolmogorov)
except:
    import sys
    print("t_HypothesisTest_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Problem parameters
    dimension = 3
    a = 7.0
    b = 0.1
    # Reference analytical values
    meanTh = a / 2
    covTh = (b ** 2 * pi ** 8) / 18.0 + \
        (b * pi ** 4) / 5.0 + (a ** 2) / 8.0 + 1.0 / 2.0
    sob_1 = [(b * pi ** 4 / 5.0 + b ** 2 * pi ** 8 / 50.0 + 1.0 / 2.0)
             / covTh, (a ** 2 / 8.0) / covTh, 0.0]
    sob_2 = [
        0.0, (b ** 2 * pi ** 8 / 18.0 - b ** 2 * pi ** 8 / 50.0) / covTh, 0.0]
    sob_3 = [0.0]
    sob_T1 = [sob_1[0] + sob_2[0] + sob_2[1] + sob_3[0], sob_1[1] + sob_2[0]
              + sob_2[2] + sob_3[0], sob_1[2] + sob_2[1] + sob_2[2] + sob_3[0]]
    sob_T2 = [sob_2[0] + sob_2[1] + sob_3[0], sob_2[0]
              + sob_2[2] + sob_3[0], sob_2[1] + sob_2[2] + sob_3[0]]
    sob_T3 = [sob_3[0]]
    # Create the Ishigami function
    inputVariables = Description(dimension)
    inputVariables[0] = "xi1"
    inputVariables[1] = "xi2"
    inputVariables[2] = "xi3"
    formula = Description(1)
    formula[0] = "sin(xi1) + (" + str(a) + \
        ") * (sin(xi2)) ^ 2 + (" + str(
            b) + ") * xi3^4 * sin(xi1)"
    model = SymbolicFunction(inputVariables, formula)

    # Create the input distribution
    distribution = ComposedDistribution([Uniform(-pi, pi)] * dimension)

    # Create the orthogonal basis
    enumerateFunction = LinearEnumerateFunction(dimension)
    productBasis = OrthogonalProductPolynomialFactory(
        [LegendreFactory()] * dimension, enumerateFunction)

    # Create the adaptive strategy
    # We can choose amongst several strategies
    # First, the most efficient (but more complex!) strategy
    listAdaptiveStrategy = list()
    degree = 6
    indexMax = enumerateFunction.getStrataCumulatedCardinal(degree)
    basisDimension = enumerateFunction.getStrataCumulatedCardinal(degree // 2)
    threshold = 1.0e-6
    listAdaptiveStrategy.append(
        CleaningStrategy(productBasis, indexMax, basisDimension, threshold, False))
    # Second, the most used (and most basic!) strategy
    listAdaptiveStrategy.append(
        FixedStrategy(productBasis, enumerateFunction.getStrataCumulatedCardinal(degree)))
    # Third, a slight enhancement with respect to the basic strategy
    listAdaptiveStrategy.append(
        SequentialStrategy(productBasis, enumerateFunction.getStrataCumulatedCardinal(degree // 2), False))

    for adaptiveStrategyIndex in range(len(listAdaptiveStrategy)):
        adaptiveStrategy = listAdaptiveStrategy[adaptiveStrategyIndex]
        # Create the projection strategy
        samplingSize = 250
        listProjectionStrategy = list()
        # We have only the LeastSquaresStrategy up to now (0.13.0) but we can choose several sampling schemes
        # Monte Carlo sampling
        listProjectionStrategy.append(
            LeastSquaresStrategy(MonteCarloExperiment(samplingSize)))
        # LHS sampling
        listProjectionStrategy.append(
            LeastSquaresStrategy(LHSExperiment(samplingSize)))
        # Low Discrepancy sequence
        listProjectionStrategy.append(LeastSquaresStrategy(
            LowDiscrepancyExperiment(SobolSequence(), samplingSize)))
        for projectionStrategyIndex in range(len(listProjectionStrategy)):
            projectionStrategy = listProjectionStrategy[
                projectionStrategyIndex]
            # Create the polynomial chaos algorithm
            maximumResidual = 1.0e-10
            algo = FunctionalChaosAlgorithm(
                model, distribution, adaptiveStrategy, projectionStrategy)
            algo.setMaximumResidual(maximumResidual)
            RandomGenerator.SetSeed(0)
            algo.run()

            # Examine the results
            result = FunctionalChaosResult(algo.getResult())
            pGrad = result.getMetaModel().parameterGradient(
                distribution.getMean())
            print("###################################")
            print(algo.getAdaptiveStrategy())
            print(algo.getProjectionStrategy())
            # print "coefficients=", result.getCoefficients()
            residuals = result.getResiduals()
            print("residuals=", residuals)
            relativeErrors = result.getRelativeErrors()
            print("relativeErrors=", relativeErrors)

            # Post-process the results
            vector = FunctionalChaosRandomVector(result)
            mean = vector.getMean()[0]
            print("mean=%.8f" % mean, "absolute error=%.10f" %
                  fabs(mean - meanTh))
            variance = vector.getCovariance()[0, 0]
            print("variance=%.8f" % variance, "absolute error=%.10f" %
                  fabs(variance - covTh))
            sensitivity = FunctionalChaosSobolIndices(result)
            for i in range(dimension):
                value = sensitivity.getSobolIndex(i)
                print("Sobol index", i, "= %.8f" %
                      value, "absolute error=%.10f" % fabs(value - sob_1[i]))
            indices = Indices(2)
            k = 0
            for i in range(dimension):
                indices[0] = i
                for j in range(i + 1, dimension):
                    indices[1] = j
                    value = sensitivity.getSobolIndex(indices)
                    print("Sobol index", indices, "=%.8f" %
                          value, "absolute error=%.10f" % fabs(value - sob_2[k]))
                    k = k + 1
            indices = Indices(3)
            indices[0] = 0
            indices[1] = 1
            indices[2] = 2
            value = sensitivity.getSobolIndex(indices)
            print("Sobol index", indices, "=%.8f" %
                  value, "absolute error=%.10f" % fabs(value - sob_3[0]))
            for i in range(dimension):
                value = sensitivity.getSobolTotalIndex(i)
                print("Sobol total index", i, "=%.8f" %
                      value, "absolute error=%.10f" % fabs(value - sob_T1[i]))
            indices = Indices(2)
            k = 0
            for i in range(dimension):
                indices[0] = i
                for j in range(i + 1, dimension):
                    indices[1] = j
                    value = sensitivity.getSobolIndex(indices)
                    print("Sobol total index", indices, "=%.8f" %
                          value, "absolute error=%.10f" % fabs(value - sob_2[k]))
                    k = k + 1
            indices = Indices(3)
            indices[0] = 0
            indices[1] = 1
            indices[2] = 2
            value = sensitivity.getSobolTotalIndex(indices)
            print("Sobol total index ", indices, "=%.8f" %
                  value, "absolute error=%.10f" % fabs(value - sob_3[0]))
            for i in range(dimension):
                value = sensitivity.getSobolGroupedIndex(i)
                print("Sobol grouped index", i, "=%.8f" %
                      value, "absolute error=%.10f" % fabs(value - sob_T1[i]))
            indices = Indices(2)
            k = 0
            for i in range(dimension):
                indices[0] = i
                for j in range(i + 1, dimension):
                    indices[1] = j
                    value = sensitivity.getSobolGroupedIndex(indices)
                    print("Sobol grouped index", indices, "=%.8f" %
                          value, "absolute error=%.10f" % fabs(value - sob_2[k]))
                    k = k + 1
            indices = Indices(3)
            indices[0] = 0
            indices[1] = 1
            indices[2] = 2
            value = sensitivity.getSobolGroupedIndex(indices)
            print("Sobol grouped index ", indices, "=%.8f" %
                  value, "absolute error=%.10f" % fabs(value - sob_3[0]))
            indices = Indices(3)
            indices[0] = 0
            indices[1] = 1
            indices[2] = 2
            value = sensitivity.getSobolGroupedTotalIndex(indices)
            print("Sobol grouped total index ", indices, "=%.8f" %
                  value, "absolute error=%.10f" % fabs(value - 1.))

except:
    import sys
    print("t_FunctionalChaos_ishigami.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
import sys

TESTPREAMBLE()

mesh = RegularGrid(0.0, 0.1, 11)


class FUNC(OpenTURNSPythonPointToFieldFunction):

    def __init__(self):
        super(FUNC, self).__init__(2, mesh, 2)
        self.setInputDescription(['R', 'S'])
        self.setOutputDescription(['T', 'U'])
        self.mesh_ = RegularGrid(0.0, 0.1, 11)

    def _exec(self, X):
        size = self.mesh_.getVerticesNumber()
        values = [Point(X) * i for i in range(size)]
        Y = Field(self.mesh_, values)
        return Y


F = FUNC()
print('in_dim=' + str(F.getInputDimension())
      + ' out_dim=' + str(F.getOutputDimension()))

print((F((10.0, 5.0))))

print((F(((10.0, 5.0), (6.0, 7.0)))))


# Instance creation
myFunc = PointToFieldFunction(F)

# Copy constructor
newFunc = PointToFieldFunction(myFunc)

print(('myFunc input dimension= ' + str(myFunc.getInputDimension())))
print(('myFunc output dimension= ' + str(myFunc.getOutputDimension())))

inPt = Point(2, 2.0)
print((repr(inPt)))

outPt = myFunc(inPt)
print((repr(outPt)))

outPt = myFunc((10.0, 11.0))
print((repr(outPt)))

inSample = Sample(10, 2)
for i in range(10):
    inSample[i] = Point((i, i))
print((repr(inSample)))

outSample = myFunc(inSample)
print((repr(outSample)))

outSample = myFunc(((100.0, 100.0), (101.0, 101.0), (102.0, 102.0)))
print((repr(outSample)))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # Instanciate one distribution object
    dim = 3
    R = CorrelationMatrix(dim)
    for i in range(dim - 1):
        R[i, i + 1] = 0.25

    copula = NormalCopula(R)
    print("Copula ", repr(copula))
    print("Copula ", copula)
    print("Mean ", repr(copula.getMean()))
    print("Covariance ", repr(copula.getCovariance()))

    # Is this copula an elliptical distribution?
    print("Elliptical distribution= ", copula.isElliptical())

    # Is this copula elliptical ?
    print("Elliptical copula= ", copula.hasEllipticalCopula())

    # Is this copula independent ?
    print("Independent copula= ", copula.hasIndependentCopula())

    # Test for realization of distribution
    oneRealization = copula.getRealization()
    print("oneRealization=", repr(oneRealization))

    # Test for sampling
    size = 10
    oneSample = copula.getSample(size)
    print("oneSample=", repr(oneSample))

    # Test for sampling
    size = 10000
    anotherSample = copula.getSample(size)
    print("anotherSample mean=", repr(anotherSample.computeMean()))
    print("anotherSample covariance=", repr(anotherSample.computeCovariance()))

    # Define a point
    point = Point(dim, 0.2)

    # Show PDF and CDF of point
    pointPDF = copula.computePDF(point)
    pointCDF = copula.computeCDF(point)
    print("Point = ", repr(point), " pdf=%.6f" %
          pointPDF, " cdf=%.6f" % pointCDF)

    # Get 50% quantile
    quantile = copula.computeQuantile(0.5)
    print("Quantile=", repr(quantile))
    print("CDF(quantile)=%.6f" % copula.computeCDF(quantile))
    # Get 95% survival function
    inverseSurvival = Point(copula.computeInverseSurvivalFunction(0.95))
    print("InverseSurvival=", repr(inverseSurvival))
    print("Survival(inverseSurvival)=%.6f" %
          copula.computeSurvivalFunction(inverseSurvival))
    print("entropy=%.6f" % copula.computeEntropy())
    # Confidence regions
    if copula.getDimension() <= 2:
        threshold = Point()
        print("Minimum volume interval=",
              copula.computeMinimumVolumeInterval(0.95, threshold))
        print("threshold=", threshold)
        beta = Point()
        levelSet = copula.computeMinimumVolumeLevelSet(0.95, beta)
        print("Minimum volume level set=", levelSet)
        print("beta=", beta)
        print("Bilateral confidence interval=",
              copula.computeBilateralConfidenceInterval(0.95, beta))
        print("beta=", beta)
        print("Unilateral confidence interval (lower tail)=",
              copula.computeUnilateralConfidenceInterval(0.95, False, beta))
        print("beta=", beta)
        print("Unilateral confidence interval (upper tail)=",
              copula.computeUnilateralConfidenceInterval(0.95, True, beta))
        print("beta=", beta)

    # Covariance and correlation
    covariance = copula.getCovariance()
    print("covariance=", covariance)
    correlation = copula.getCorrelation()
    print("correlation=", correlation)
    spearman = copula.getSpearmanCorrelation()
    print("spearman=", spearman)
    kendall = copula.getKendallTau()
    print("kendall=", kendall)
    x = 0.6
    y = [0.2]*(dim-1)
    print("conditional PDF=%.6f" % copula.computeConditionalPDF(x, y))
    print("conditional CDF=%.6f" % copula.computeConditionalCDF(x, y))
    print("conditional quantile=%.6f" %
          copula.computeConditionalQuantile(x, y))
    pt = Point([0.1 * i + 0.05 for i in range(dim)])
    print("sequential conditional PDF=",
          copula.computeSequentialConditionalPDF(point))
    resCDF = copula.computeSequentialConditionalCDF(pt)
    print("sequential conditional CDF(", pt, ")=", resCDF)
    print("sequential conditional quantile(", resCDF, ")=",
          copula.computeSequentialConditionalQuantile(resCDF))

    # Extract the marginals
    for i in range(dim):
        margin = copula.getMarginal(i)
        print("margin=", repr(margin))
        print("margin PDF=%.6f" % margin.computePDF(Point(1, 0.25)))
        print("margin CDF=%.6f" % margin.computeCDF(Point(1, 0.25)))
        print("margin quantile=", repr(margin.computeQuantile(0.95)))
        print("margin realization=", repr(margin.getRealization()))

    # Extract a 2-D marginal
    indices = Indices(2, 0)
    indices[0] = 1
    indices[1] = 0
    print("indices=", repr(indices))
    margins = copula.getMarginal(indices)
    print("margins=", repr(margins))
    print("margins PDF=%.6f" % margins.computePDF(Point(2, 0.25)))
    print("margins CDF=%.6f" % margins.computeCDF(Point(2, 0.25)))
    quantile = Point(margins.computeQuantile(0.95))
    print("margins quantile=", repr(quantile))
    print("margins CDF(qantile)=%.6f" % margins.computeCDF(quantile))
    print("margins realization=", repr(margins.getRealization()))

    # Creation of the correlation matrix from a Spearman correlation matrix
    spearman = CorrelationMatrix(dim)
    for i in range(1, dim):
        spearman[i, i - 1] = 0.25

    correlation = NormalCopula.GetCorrelationFromSpearmanCorrelation(spearman)
    print("Normal copula correlation=", repr(correlation),
          " from the Spearman correlation=", repr(spearman))

    # computeProbability
    spearman_corr = CorrelationMatrix([[1., .74], [.74, 1.]])
    copula = NormalCopula(
        NormalCopula.GetCorrelationFromSpearmanCorrelation(spearman_corr))
    interval = Interval([0.958722, 0.902063], [1., 1.])
    prob = copula.computeProbability(interval)
    print("prob=%.6f" % prob)

    print(NormalCopula(1).getParametersCollection())
except:
    import sys
    print("t_NormalCopula_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import openturns.testing as ottest

dim = 3

factoryCollection = [ot.FourierSeriesFactory()] * dim

rank = 2
nk = [10] * dim
tensorEval = ot.CanonicalTensorEvaluation(factoryCollection, nk, rank)

for r in range(rank):
    for j in range(dim):
        tensorEval.setCoefficients(r, j, [1.0] * nk[j])

f = ot.Function(tensorEval)
print(tensorEval)

x = [1.0] * dim
print(x)

y = f(x)
print(y)

ottest.assert_almost_equal(y, [-1.45674])

tensorEval.setRank(1)
f = ot.Function(tensorEval)
print(tensorEval)

y = f(x)
print(y)

ottest.assert_almost_equal(y, [-0.728368])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

size = 100
dim = 2

sampleX = Normal(dim).getSample(size)
sampleX0 = sampleX.getMarginal(0)
sampleY = sampleX.getMarginal(1)
sampleZ = SymbolicFunction("x", "exp(x)")(sampleX0)
# Spearman Test : test if two samples have a zero rank correlation
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

print("Spearman=", HypothesisTest.Spearman(sampleY, sampleZ, 0.10))

# Full Spearman Test : collection of tests of zero correlation between the 1D marginals of a sample and another 1D sample
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

print("FullSpearmanXZ=", HypothesisTest.FullSpearman(sampleX, sampleZ, 0.10))
print("FullSpearmanYY=", HypothesisTest.FullSpearman(sampleY, sampleY, 0.10))

# Partial Spearman Test : collection of tests of zero correlation between a selection of the 1D marginals of a sample and another 1D sample
# H0 = zero rank correlation
# Test = True <=> zero rank correlation
# p-value threshold : probability of the critical region, ie P_H0(reject H0)
# p-value : P_H0(test statistics > observed test statistics)
# Test = True <=> p-value > p-value threshold

selection = [0]
# The three tests must be equal
print("PartialSpearmanX0Y=", HypothesisTest.PartialSpearman(
    sampleX, sampleY, selection, 0.10))
print("SpearmanX0Y=", HypothesisTest.Spearman(sampleX0, sampleY, 0.10))
print("FullSpearmanX0Y=", HypothesisTest.FullSpearman(sampleX0, sampleY, 0.10))

print("PartialSpearmanXY=", HypothesisTest.PartialSpearman(
    sampleX, sampleY, selection, 0.10))



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

# Instance creation
myFunc = ot.SymbolicFunction(
    ['x1', 'x2'], ['x1*sin(x2)', 'cos(x1+x2)', '(x2+1)*exp(x1-2*x2)'])

# Copy constructor
newFunc = ot.Function(myFunc)

print("myFunc=" + repr(myFunc))
print("myFunc input parameter(s)=")
for i in range(myFunc.getInputDimension()):
    print(myFunc.getInputDescription()[i])
print("myFunc output parameter(s) and marginal(s)=")
for i in range(myFunc.getOutputDimension()):
    print(myFunc.getOutputDescription()[i])
    print("Marginal function", i, "=", repr(myFunc.getMarginal(i)))

try:
    print(ot.Function() == None)
except TypeError:
    # swig<4
    print(False)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import os
import inspect

ot.TESTPREAMBLE()
ot.Log.Show(ot.Log.NONE)

# find all instanciable classes
instanciables = []
for name, obj in inspect.getmembers(ot):
    if inspect.isclass(obj):
        cn = obj.__name__
        if '_' in cn or 'Interface' in cn or 'Pointer' in cn:
            continue
        try:
            instance = obj()
            print (obj.__name__, 'OK')
            instanciables.append(obj)
        except Exception:
            print (obj.__name__, 'FAIL')

# save / load
fileName = 'myStudy.xml'
for class_ in instanciables:
    myStudy = ot.Study()
    myStudy.setStorageManager(ot.XMLStorageManager(fileName))

    print (class_.__name__)
    try:
        instance = class_()
        myStudy.add(class_.__name__, instance)
        myStudy.save()
        myStudy = ot.Study()
        myStudy.setStorageManager(ot.XMLStorageManager(fileName))
        myStudy.load()
        os.remove(fileName)
        instance = class_()
        myStudy.fillObject(class_.__name__, instance)
        print (class_.__name__, 'OK')
    except Exception as exc:
        print('--', class_.__name__, exc)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

dim = 2
distribution = ot.Normal(dim)
#
# Case 1: composite random vector based event
#

# The input vector
X = ot.RandomVector(distribution)
# The model: the identity function
inVars = ot.Description(dim)
for i in range(dim):
    inVars[i] = "x" + str(i)
model = ot.SymbolicFunction(inVars, inVars)
# The output vector
Y = ot.CompositeRandomVector(model, X)
# The domain: [0, 1]^dim
domain = ot.Interval(dim)
# The event
event = ot.DomainEvent(Y, domain)

print("sample=", event.getSample(10))

#
# Case 2: process based event
#

# The input process
X = ot.WhiteNoise(distribution)
# The domain: [0, 1]^dim
domain = ot.Interval(dim)
# The event
event = ot.ProcessEvent(X, domain)

print("sample=", event.getSample(10))

# 3. from distribution
antecedent = ot.RandomVector(ot.Normal(2))
domain = ot.LevelSet(ot.SymbolicFunction(
    ['x', 'y'], ['x^2+y^2']), ot.Less(), 1.0)
event = ot.DomainEvent(antecedent, domain)
print('sample=', event.getSample(10))



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import time

ticksCollection = [[0.0, 1.0, 3.0, 6.0], [1.0, 2.0, 3.0]]

kind = [ot.MixedHistogramUserDefined.CONTINUOUS,
        ot.MixedHistogramUserDefined.DISCRETE]
probabilityTable = [0.05, 0.1, 0.15, 0.2, 0.25, 0.3, 0.35, 0.4, 0.45]
distribution = ot.MixedHistogramUserDefined(
    ticksCollection, kind, probabilityTable)
distribution.setDescription(["A", "B"])
print("Distribution ", repr(distribution))
print("Distribution ", distribution)

# Get mean and covariance
print("Mean= ", repr(distribution.getMean()))
print("Covariance= ", repr(distribution.getCovariance()))

# Is this distribution elliptical ?
print("Elliptical = ", distribution.isElliptical())

# Test for realization of distribution
oneRealization = distribution.getRealization()
print("oneRealization=", repr(oneRealization))

# Test for sampling
size = 10000
oneSample = distribution.getSample(size)
print("oneSample first=", repr(
    oneSample[0]), " last=", repr(oneSample[size - 1]))
print("mean=", repr(oneSample.computeMean()))
print("covariance=", repr(oneSample.computeCovariance()))

# Define a point
point = ot.Point(distribution.getDimension(), 1.0)
print("Point= ", repr(point))

# Show PDF and CDF of point
eps = 1e-5

# PDF value
LPDF = distribution.computeLogPDF(point)
print("log pdf=%.6f" % LPDF)
PDF = distribution.computePDF(point)
print("pdf     =%.6f" % PDF)

# derivative of the PDF with regards the parameters of the distribution
CDF = distribution.computeCDF(point)
print("cdf=%.6f" % CDF)
CCDF = distribution.computeComplementaryCDF(point)
print("ccdf=%.6f" % CCDF)
# quantile
quantile = distribution.computeQuantile(0.95)
print("quantile=", repr(quantile))
print("cdf(quantile)=%.6f" % distribution.computeCDF(quantile))
mean = distribution.getMean()
print("mean=", repr(mean))
standardDeviation = distribution.getStandardDeviation()
print("standard deviation=", repr(standardDeviation))
skewness = distribution.getSkewness()
print("skewness=", repr(skewness))
kurtosis = distribution.getKurtosis()
print("kurtosis=", repr(kurtosis))
for i in range(6):
    print("standard moment n=", i, " value=",
          distribution.getStandardMoment(i))
print("Standard representative=", distribution.getStandardRepresentative())


# higher dimension
dim = 8
ticksCollection = [[0.0, 1.0, 3.0, 6.0]]*dim
kind = [ot.MixedHistogramUserDefined.DISCRETE] * dim
kind[dim-2] = ot.MixedHistogramUserDefined.CONTINUOUS
probabilityTable = ot.RandomGenerator.Generate(4**(dim-1)*3)
x = [3.0]*dim
x[dim-2] = 5.0
bench = 0
distribution = ot.MixedHistogramUserDefined(
    ticksCollection, kind, probabilityTable)
t0 = time.time()
print('pdf=%.6g' % distribution.computePDF(x))
t1 = time.time()
#print('pdf t=%.6g' % (t1-t0))
t0 = time.time()
print('cdf=%.6g' % distribution.computeCDF(x))
t1 = time.time()
#print('cdf t=%.6g' % (t1-t0))

# test default ctor
ot.MixedHistogramUserDefined().getSample(5)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    matrix1 = CorrelationMatrix(2)
    matrix1.setName("matrix1")
    print("matrix1 = ", repr(matrix1))

    p = Point((1., 3.))
    print("matrix1*", p, "=", matrix1 * p)

    s = 3.
    print("matrix1*", s, "=", matrix1 * s)
    print(s, "*matrix1=", s * matrix1)
    print("matrix1/", s, "=", matrix1 / s)

    Vars = ['X', 'Y', 'Z']
    Map = {}
    Map['X'] = {}
    Map['X']['Y'] = 0.1
    Map['X']['Z'] = 0.7
    Map['Y'] = {}
    Map['Y']['Z'] = 0.8
    matrix2 = getCorrelationMatrixFromMap(Vars, Map)
    print(matrix2)

except:
    import traceback
    traceback.print_exc()



#! /usr/bin/env python

from __future__ import print_function
import os

width = 40

# check that python can load OpenTURNS module
print(' 1: import openturns'.ljust(width), end=' ')
try:
    import openturns as ot
    print('ok')
except ImportError:
    print('n/a')
    raise

# check that python can find the Viewer module
# If it fails, check that matplotlib package is installed
print(' 2: plotting (matplotlib)'.ljust(width), end=' ')
try:
    import openturns.viewer
    print('ok')
except ImportError:
    print('n/a')

# check that OpenTURNS can run R
# It should produce a file named testDraw.png
print(' 3: legacy plotting (R)'.ljust(width), end=' ')
try:
    graph = ot.Normal().drawPDF()
    fname = 'testDraw.png'
    graph.draw(fname)
    os.remove(fname)
    print('ok')
except:
    print('n/a')

# check XML support
print(' 4: serialization (LibXML2)'.ljust(width), end=' ')
try:
    storageManager = ot.XMLStorageManager('myFile.xml')
    print('ok')
except:
    print('n/a')

# check that math parser is available
print(' 5: legacy symbolic function (muParser)'.ljust(width), end=' ')
try:
    ot.ResourceMap.Set("SymbolicParser-Backend", "MuParser")
    f = ot.SymbolicFunction(['x1', 'x2'], ['x1+x2'])
    print('ok')
except:
    print('n/a')

# check that hmat library was found
print(' 6: h-matrix (hmat-oss)'.ljust(width), end=' ')
print('ok' if ot.HMatrixFactory.IsAvailable() else 'n/a')

# check that spectra library was found
print(' 7: iterative SVD (Spectra)'.ljust(width), end=' ')
ot.ResourceMap.SetAsString(
    "KarhunenLoeveP1Algorithm-EigenvaluesSolver", "SPECTRA")
mesh = ot.IntervalMesher([9]).build(ot.Interval(-1.0, 1.0))
cov1D = ot.AbsoluteExponential([1.0])
algo = ot.KarhunenLoeveP1Algorithm(mesh, cov1D, 1e-3)
try:
    algo.run()
    result = algo.getResult()
    lambd = result.getEigenValues()
    print('ok')
except:
    print('n/a')

# check that TBB library was found
print(' 8: multithreading (TBB)'.ljust(width), end=' ')
print('ok' if ot.TBB.IsAvailable() else 'n/a')

# check that psutil was found
print(' 9: process control (psutil)'.ljust(width), end=' ')
try:
    import psutil
    print('ok')
except ImportError:
    print('n/a')

# check that nlopt library was found
print('10: optimization (NLopt)'.ljust(width), end=' ')
print('ok' if ot.NLopt.IsAvailable() else 'n/a')

# check that cminpack was found
print('11: optimization (CMinpack)'.ljust(width), end=' ')
print('ok' if ot.CMinpack.IsAvailable() else 'n/a')

# check that ceres was found
print('12: optimization (Ceres Solver)'.ljust(width), end=' ')
print('ok' if ot.Ceres.IsAvailable() else 'n/a')

# check that dlib was found
print('13: optimization (Dlib)'.ljust(width), end=' ')
print('ok' if ot.Dlib.IsAvailable() else 'n/a')

# check that dlib was found
print('14: optimization (Bonmin)'.ljust(width), end=' ')
print('ok' if ot.Bonmin.IsAvailable() else 'n/a')

# check that ipopt was found
print('15: optimization (Ipopt)'.ljust(width), end=' ')
print('ok' if ot.Ipopt.IsAvailable() else 'n/a')



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import math as m
import os


class UniformNdPy(ot.PythonDistribution):

    def __init__(self, a=[0.0], b=[1.0]):
        super(UniformNdPy, self).__init__(len(a))
        if len(a) != len(b):
            raise ValueError('Invalid bounds')
        for i in range(len(a)):
            if a[i] > b[i]:
                raise ValueError('Invalid bounds')
        self.a = a
        self.b = b
        self.factor = 1.0
        for i in range(len(a)):
            self.factor *= (b[i] - a[i])

    def getRange(self):
        return ot.Interval(self.a, self.b, [True] * len(self.a), [True] * len(self.a))

    def getRealization(self):
        X = []
        for i in range(len(self.a)):
            X.append(
                self.a[i] + (self.b[i] - self.a[i]) * ot.RandomGenerator.Generate())
        return X

    def getSample(self, size):
        X = []
        for i in range(size):
            X.append(self.getRealization())
        return X

    def computeCDF(self, X):
        prod = 1.0
        for i in range(len(self.a)):
            if X[i] < self.a[i]:
                return 0.0
            prod *= (min(self.b[i], X[i]) - self.a[i])
        return prod / self.factor

    def computePDF(self, X):
        for i in range(len(self.a)):
            if X[i] < self.a[i]:
                return 0.0
            if X[i] > self.b[i]:
                return 0.0
        return 1.0 / self.factor

    def getRoughness(self):
        return 42.0

    def getMean(self):
        mu = []
        for i in range(len(self.a)):
            mu.append(0.5 * (self.a[i] + self.b[i]))
        return mu

    def getStandardDeviation(self):
        stdev = []
        for i in range(len(self.a)):
            stdev.append((self.b[i] - self.a[i]) / m.sqrt(12.))
        return stdev

    def getSkewness(self):
        return [0.] * len(self.a)

    def getKurtosis(self):
        return [1.8] * len(self.a)

    def getStandardMoment(self, n):
        if n % 2 == 1:
            return [0.] * len(self.a)
        return [1. / (n + 1.)] * len(self.a)

    def getMoment(self, n):
        return [-0.1 * n] * len(self.a)

    def getCenteredMoment(self, n):
        return [0.] * len(self.a)

    def computeCharacteristicFunction(self, x):
        if len(self.a) > 1:
            raise ValueError('dim>1')
        ax = self.a[0] * x
        bx = self.b[0] * x
        return (m.sin(bx) - m.sin(ax) + 1j * (m.cos(ax) - m.cos(bx))) / (bx - ax)

    def isElliptical(self):
        return (len(self.a) == 1) and (self.a[0] == -self.b[0])

    def isCopula(self):
        for i in range(len(self.a)):
            if self.a[i] != 0.0:
                return False
            if self.b[i] != 1.0:
                return False
        return True

    def getMarginal(self, indices):
        subA = []
        subB = []
        for i in indices:
            subA.append(self.a[i])
            subB.append(self.b[i])
        py_dist = UniformNdPy(subA, subB)
        return ot.Distribution(py_dist)

    def computeQuantile(self, prob, tail=False):
        p = 1.0 - prob if tail else prob
        quantile = self.a
        for i in range(len(self.a)):
            quantile[i] += p * (self.b[i] - self.a[i])
        return quantile

    def getParameter(self):
        param = list(self.a)
        param.extend(self.b)
        return param

    def getParameterDescription(self):
        paramDesc = ['a_' + str(i) for i in range(len(self.a))]
        paramDesc.extend(['b_' + str(i) for i in range(len(self.a))])
        return paramDesc

    def setParameter(self, parameter):
        dim = len(self.a)
        for i in range(dim):
            self.a[i] = parameter[i]
            self.b[i] = parameter[dim + i]


myDist = ot.Distribution(UniformNdPy([0.0] * 2, [2.0] * 2))

st = ot.Study()
fileName = 'PyDIST.xml'
st.setStorageManager(ot.XMLStorageManager(fileName))

st.add("myDist", myDist)
st.save()

print('saved dist=', myDist)

dist = ot.Distribution()

st = ot.Study()
st.setStorageManager(ot.XMLStorageManager(fileName))

st.load()

st.fillObject("myDist", dist)
print('loaded dist=', dist)
os.remove(fileName)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import os

ot.TESTPREAMBLE()

grids = [ot.RegularGrid(1.0, 0.1, 20), ot.RegularGrid(-3.0, 0.1, 20),
         ot.RegularGrid(3.0, -0.1, 20), ot.RegularGrid(-1.0, -0.1, 20),
         ot.RegularGrid(-1.0, 0.13, 20), ot.RegularGrid(1.0, -0.13, 20)]

for regularGrid in grids:
    lowerBound = regularGrid.getLowerBound()[0]
    upperBound = regularGrid.getUpperBound()[0]
    n = regularGrid.getSimplicesNumber()
    print("regularGrid=", regularGrid, "lowerBound=",
          lowerBound, "upperBound=", upperBound, n, "simplices")
    algo = ot.RegularGridEnclosingSimplex(regularGrid)

    ot.RandomGenerator.SetSeed(0)
    test = ot.Sample(ot.Uniform(lowerBound - 0.2 * (upperBound - lowerBound),
                                upperBound + 0.2 * (upperBound - lowerBound)).getSample(1000))

    vertices = regularGrid.getVertices()
    for vertex in test:
        index = algo.query(vertex)
        x = vertex[0]
        if x < lowerBound or x > upperBound:
            if index < n:
                print(
                    "Point", x, "should be outside but query returned index", index)
                os.exit(1)
        else:
            if index >= n:
                print(
                    "Point", x, "should be inside but query returned index", index)
                os.exit(1)

            found, coordinates = regularGrid.checkPointInSimplexWithCoordinates(
                vertex, index)
            if not found:
                print("Wrong simplex found for", x,
                      "(index=", index, ") barycentric coordinates=", coordinates)
                os.exit(1)
            if coordinates[0] < 0.0 or coordinates[0] > 1.0 or coordinates[1] < 0.0 or coordinates[1] > 1.0:
                print("Wrong barycentric coordinates found found for", x,
                      "(index=", index, ") barycentric coordinates=", coordinates)
                os.exit(1)
            if abs(x - coordinates[0] * vertices[index, 0] - coordinates[1] * vertices[index + 1, 0]) > 1.e-10:
                print("Wrong barycentric coordinates found found for", x,
                      "(index=", index, ") barycentric coordinates=", coordinates)
                os.exit(1)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot
import openturns.testing as ott
from math import sqrt

ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)


mu = [0.0] * 4
sigma = [1.0] * 4
a = [-4., -1., 1., 3.]
b = [4., 4., 2., 6.]

for i in range(4):
    ot.PlatformInfo.SetNumericalPrecision(1 if i == 2 else 2)
    distribution = ot.TruncatedNormal(mu[i], sigma[i], a[i], b[i])
    size = 10000
    sample = distribution.getSample(size)
    factory = ot.TruncatedNormalFactory()
    estimatedDistribution = factory.build(sample)
    print("distribution=", repr(distribution))
    print("Estimated distribution=", repr(estimatedDistribution))
    estimatedDistribution = factory.build()
    print("Default distribution=", estimatedDistribution)
    estimatedDistribution = factory.build(
        distribution.getParameter())
    print("Distribution from parameters=", estimatedDistribution)
    estimatedTruncatedNormal = factory.buildAsTruncatedNormal(sample)
    print("TruncatedNormal          =", distribution)
    print("Estimated TruncatedNormal=", estimatedTruncatedNormal)
    estimatedTruncatedNormal = factory.buildAsTruncatedNormal()
    print("Default TruncatedNormal=", estimatedTruncatedNormal)
    estimatedTruncatedNormal = factory.buildAsTruncatedNormal(
        distribution.getParameter())
    print("TruncatedNormal from parameters=", estimatedTruncatedNormal)
    # Various estimators
    estimatedDistribution = factory.build(sample)
    print('Estimated distribution with default estimator =', estimatedDistribution)
    estimatedDistribution = factory.buildMethodOfMoments(sample)
    print('Estimated distribution with moments=', estimatedDistribution)
    estimatedDistribution = factory.buildMethodOfLikelihoodMaximization(sample)
    print('Estimated distribution with likelihoodMax.=', estimatedDistribution)
sample = [[0.0]] * size
estimatedDistribution = factory.build(sample)
print("Estimated distribution with default estimator=",
      repr(estimatedDistribution))
sample = [[1.0]] * size
estimatedDistribution = factory.build(sample)
print("Estimated distribution with default estimator=",
      repr(estimatedDistribution))

# Build method of moments
print("Build method of moments")
size = 10000
distribution = ot.TruncatedNormal(2.0, 3.0, -1.0, 4.0)
sample = distribution.getSample(size)
factory = ot.TruncatedNormalFactory()
estimatedTN = factory.buildMethodOfMoments(sample)
exact_mu = sample.computeMean()[0]
exact_sigma2 = sample.computeCovariance()[0, 0]
exact_a = sample.getMin()[0]
exact_b = sample.getMax()[0]
computed_mu = estimatedTN.getMean()[0]
computed_sigma2 = estimatedTN.getCovariance()[0, 0]
computed_a = estimatedTN.getA()
computed_b = estimatedTN.getB()
ott.assert_almost_equal(exact_mu, computed_mu, 1.e-2, 0.0)
ott.assert_almost_equal(exact_sigma2, computed_sigma2, 1.e-3, 0.0)
ott.assert_almost_equal(exact_a, computed_a, 0.0, 10.0 / size)
ott.assert_almost_equal(exact_b, computed_b, 0.0, 10.0 / size)

# Build method of likelihood maximization
print("Build method of likelihood maximization")
size = 10000
distribution = ot.TruncatedNormal(2.0, 3.0, -1.0, 4.0)
sample = distribution.getSample(size)
factory = ot.TruncatedNormalFactory()
estimatedTN = factory.buildMethodOfLikelihoodMaximization(sample)
exact_mu = distribution.getMu()
exact_sigma = distribution.getSigma()
exact_a = distribution.getA()
exact_b = distribution.getB()
computed_mu = estimatedTN.getMu()
computed_sigma = estimatedTN.getSigma()
computed_a = estimatedTN.getA()
computed_b = estimatedTN.getB()
ott.assert_almost_equal(exact_mu, computed_mu, 0.0, 20.0 / sqrt(size))
ott.assert_almost_equal(exact_sigma, computed_sigma, 0.0, 20.0 / sqrt(size))
ott.assert_almost_equal(exact_a, computed_a, 0.0, 20.0 / size)
ott.assert_almost_equal(exact_b, computed_b, 0.0, 20.0 / size)

# A tricky case
print("A tricky case")
data = [0.6852, 0.9349, 0.5884, 1.727, 1.581, 0.3193, -0.5701, 1.623, 2.210,
        -0.3440, -0.1646]
sample = ot.Sample(data, 1)
factory = ot.TruncatedNormalFactory()
estimatedTN = factory.buildMethodOfMoments(sample)
exact_mu = -1.0
exact_sigma = 2.5
exact_a = 2.
exact_b = 3.
computed_mu = estimatedTN.getMu()
computed_sigma = estimatedTN.getSigma()
computed_a = estimatedTN.getA()
computed_b = estimatedTN.getB()
ott.assert_almost_equal(exact_mu, computed_mu, 0.0, 2.0)
ott.assert_almost_equal(exact_sigma, computed_sigma, 0.0, 1.0)
ott.assert_almost_equal(exact_a, computed_a, 0.0, 3.0)
ott.assert_almost_equal(exact_b, computed_b, 0.0, 3.0)



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.PlatformInfo.SetNumericalPrecision(3)
distribution = ot.Pareto(3.3, 7.5, 0.0)
print('distribution=', distribution)
sample = distribution.getSample(1000)
factory = ot.LeastSquaresDistributionFactory(ot.Pareto())
factory.setKnownParameter([0.0], [2])
inf_distribution = factory.build(sample)
print('estimated distribution=', inf_distribution)



from __future__ import print_function
import openturns as ot
from math import sin, pi
# import matplotlib
# matplotlib.use('Agg')
# import matplotlib.pylab as plt

# Definition of the Ishigami function
dimension = 3
a = 7.0
b = 0.1
input_variables = ['xi1', 'xi2', 'xi3', 'a', 'b']
formula = ['sin(xi1) + a * (sin(xi2)) ^ 2 + b * xi3^4 * sin(xi1)']
full = ot.SymbolicFunction(input_variables, formula)
ishigami_model = ot.ParametricFunction(full, [3, 4], [a, b])

#  Generating a design of size
N = 150
# Considering independent Uniform distributions of dimension 3
# Bounds are (-pi,pi), (-pi,pi) and (-pi,pi)
distribution = ot.ComposedDistribution([ot.Uniform(-pi, pi)] * dimension)
bounds = distribution.getRange()
# Random LHS
lhs = ot.LHSExperiment(distribution, N)
lhs.setAlwaysShuffle(True)  # randomized
# Fixing C2 crit
space_filling = ot.SpaceFillingC2()
# Defining a temperature profile
temperatureProfile = ot.GeometricProfile()
# Pre conditionning : generate an optimal design with MC
nSimu = 100
algo = ot.MonteCarloLHS(lhs, nSimu, space_filling)
initialDesign = algo.generate()
result = algo.getResult()

print('initial design pre-computed. Performing SA optimization...')
# Use of initial design
algo = ot.SimulatedAnnealingLHS(
    initialDesign, distribution, temperatureProfile, space_filling)
# Retrieve optimal design
input_database = algo.generate()

result = algo.getResult()

print('initial design computed')

# Response of the model
print('sampling size = ', N)
output_database = ishigami_model(input_database)

# Learning input/output
# Usual chaos meta model
enumerate_function = ot.HyperbolicAnisotropicEnumerateFunction(dimension)
orthogonal_basis = ot.OrthogonalProductPolynomialFactory(
    dimension * [ot.LegendreFactory()], enumerate_function)
basis_size = 100
# Initial chaos algorithm
adaptive_strategy = ot.FixedStrategy(orthogonal_basis, basis_size)
# ProjectionStrategy ==> Sparse
fitting_algorithm = ot.KFold()
approximation_algorithm = ot.LeastSquaresMetaModelSelectionFactory(
    ot.LARS(), fitting_algorithm)
projection_strategy = ot.LeastSquaresStrategy(
    input_database, output_database, approximation_algorithm)
print('Surrogate model...')
distribution_ishigami = ot.ComposedDistribution(
    dimension * [ot.Uniform(-pi, pi)])
algo_pc = ot.FunctionalChaosAlgorithm(
    input_database, output_database, distribution_ishigami, adaptive_strategy, projection_strategy)
algo_pc.run()
chaos_result = algo_pc.getResult()
print('Surrogate model computed')

# Validation
lhs_validation = ot.LHSExperiment(distribution_ishigami, 100)
input_validation = lhs_validation.generate()
output_validation = ishigami_model(input_validation)
# Chaos model evaluation
output_metamodel_sample = chaos_result.getMetaModel()(input_validation)

# Cloud validation ==> change from matplotlib to pure OT
# fig = plt.figure()
# plt.plot(output_validation, output_validation, 'b-', label='Model')
# plt.plot(output_validation, output_metamodel_sample, 'r.', label='SM')
# plt.title('Surrogate Model validation - Ishigami use case')
# plt.savefig('ishigami_model_validation.png')



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

ot.TESTPREAMBLE()

# First, build two functions from R^3->R
inVar = ['x1', 'x2', 'x3']
formula = ['x1^3 * sin(x2 + 2.5 * x3) - (x1 + x2)^2 / (1.0 + x3^2)']
functions = []
functions.append(ot.SymbolicFunction(inVar, formula))
formula = ['exp(-x1 * x2 + x3) / cos(1.0 + x2 * x3 - x1)']
functions.append(ot.SymbolicFunction(inVar, formula))
# Second, build the weights
coefficients = [0.3, 2.9]
# Third, build the function
myFunction = ot.LinearCombinationFunction(functions, coefficients)
inPoint = ot.Point([1.2, 2.3, 3.4])
print('myFunction=', myFunction)
print('Value at ', inPoint, '=', myFunction(inPoint))
print('Gradient at ', inPoint, '=', myFunction.gradient(inPoint))
print('Hessian at ', inPoint, '=', myFunction.hessian(inPoint))



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    chebychev = ChebychevFactory()
    print("chebychev=", chebychev)
    for i in range(10):
        print("chebychev(", i, ")=", chebychev.build(i))
    roots = chebychev.getRoots(10)
    print("chebychev(10) roots=", repr(roots))
    nodes, weights = chebychev.getNodesAndWeights(10)
    print("chebychev(10) nodes=", nodes, "and weights=", weights)
except:
    import sys
    print("t_ChebychevFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot


f = ot.SymbolicFunction("x", "x^2")
f = ot.MemoizeFunction(f)
f.disableHistory()
print(f)
size = 4
inputSample = ot.Sample(size, 1)
for i in range(size):
    inputSample[i, 0] = i
outputSample = f(inputSample)
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
f.enableHistory()
outputSample = f(inputSample)
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
f.clearHistory()
print("Is history enabled for f? ", f.isHistoryEnabled())
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())
# Perform the computation twice
outputSample = f(inputSample)
outputSample = f(inputSample)
print("input history=", f.getInputHistory())
print("output history=", f.getOutputHistory())

mem = ot.MemoizeFunction(ot.SymbolicFunction(
    ['x1', 'x2'], ['x1+x2', '3*(x1+x2)']))
mem([1, 2])
mem2 = ot.MemoizeFunction(mem.getMarginal(1))
mem2([1, 2])
print('mem2.in', mem2.getCacheInput())
print('mem2.out', mem2.getCacheOutput())
print('mem2.hits', mem2.getCacheHits())

# check getCacheInput/getCacheOutput order
for i in range(5):
    mem([i+2, i+3])
assert mem(mem.getCacheInput()) == mem.getCacheOutput(), "wrong order"
print('ok')



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    # TEST NUMBER ZERO : DEFAULT CONSTRUCTOR AND STRING CONVERTER */
    print("test number zero : default constructor and string converter")

    # Default constructor */
    squareMatrix0 = SquareMatrix()

    # String converter */
    print("squareMatrix0 = ", repr(squareMatrix0))

    # TEST NUMBER ONE : CONSTRUCTOR WITH SIZE, OPERATOR() AND STRING CONVERTER
    # */
    print(
        "test number one : constructor with size, operator() and string converter")

    # Constructor with size */
    squareMatrix1 = SquareMatrix(2)

    # Check operator() methods */
    squareMatrix1[0, 0] = 1.
    squareMatrix1[1, 0] = 2.
    squareMatrix1[0, 1] = 3.
    squareMatrix1[1, 1] = 4.

    # String converter */
    print("squareMatrix1 = ", repr(squareMatrix1))

    # TEST NUMBER TWO : COPY CONSTRUCTOR AND STRING CONVERTER */
    print("test number two : copy constructor and string converter")

    # Copy constructor */
    squareMatrix2 = SquareMatrix(squareMatrix1)

    # String converter */
    print("squareMatrix2 = ", repr(squareMatrix2))

    # TEST NUMBER THREE : GET DIMENSIONS METHODS */
    print("test number three : get dimensions methods")

    # Get dimension methods */
    print("squareMatrix1's nbRows = ", squareMatrix1.getNbRows())
    print("squareMatrix1's nbColumns = ", squareMatrix1.getNbColumns())

    # TEST NUMBER FIVE : ASSIGNMENT METHOD */
    print("test number five : assignment method")

    # Assignment method */
    # non sense with python

    # TEST NUMBER SIX : TRANSPOSITION METHOD */
    print("test number six : transposition method")

    # Check transpose method */
    squareMatrix4 = squareMatrix1.transpose()
    print("squareMatrix1 transposed = ", repr(squareMatrix4))
    print("squareMatrix1 transposed = ", repr(squareMatrix1.transpose()))

    # TEST NUMBER SEVEN : ADDITION METHOD */
    print("test number seven : addition method")

    # Check addition method : we check the operator and the symmetry of the
    # operator, thus testing the comparison operator */
    sum1 = squareMatrix1 + squareMatrix4
    sum2 = squareMatrix4 + squareMatrix1
    print("sum1 = ", repr(sum1))
    print("sum2 = ", repr(sum2))
    print("sum1 equals sum2 = ", (sum1 == sum2))

    # TEST NUMBER EIGHT : SUBTRACTION METHOD */
    print("test number eight : subtraction method")

    # Check subtraction method */
    diff = squareMatrix1 - squareMatrix4
    print("diff = ", repr(diff))

    # TEST NUMBER NINE : MATRIX MULTIPLICATION METHOD */
    print("test number nine : matrix multiplication method")

    # Check multiplication method */
    prod = squareMatrix1 * squareMatrix4
    print("prod = ", repr(prod))

    # TEST NUMBER TEN : MULTIPLICATION WITH A NUMERICAL POINT METHOD */
    print("test number ten : multiplication with a numerical point method")

    # Create the numerical point */
    pt = Point()
    pt.add(1.)
    pt.add(2.)
    print("pt = ", repr(pt))

    # Check the product method */
    ptResult = squareMatrix1 * pt
    print("ptResult = ", repr(ptResult))

    # TEST NUMBER ELEVEN : MULTIPLICATION AND DIVISION BY A NUMERICAL SCALAR
    # METHODS */
    print(
        "test number eleven : multiplication and division by a numerical scalar methods")

    # Check the multiplication method */
    s = 3.
    scalprod1 = squareMatrix1 * s
    # bug PYTHON scalprod2 = s * matrix1
    scalprod2 = squareMatrix1 * s
    print("scalprod1 = ", repr(scalprod1))
    print("scalprod2 = ", repr(scalprod2))
    print("scalprod1 equals scalprod2 = ", (scalprod1 == scalprod2))

    # Check the division method */
    scaldiv1 = squareMatrix1 / s
    scaldiv2 = squareMatrix1 / s
    print("scaldiv1 = ", repr(scaldiv1))
    print("scaldiv2 = ", repr(scaldiv2))
    print("scaldiv1 equals scaldiv2 = ", (scaldiv1 == scaldiv2))

    # TEST NUMBER TWELVE : ISEMPTY METHOD */
    print("test number twelve : isEmpty method")

    # Check method isEmpty */
    squareMatrix5 = SquareMatrix()
    print("squareMatrix0 is empty = ", squareMatrix0.isEmpty())
    print("squareMatrix1 is empty = ", squareMatrix1.isEmpty())
    print("squareMatrix5 is empty = ", squareMatrix5.isEmpty())

except:
    import sys
    print("t_SquareMatrix_std.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python
from __future__ import print_function
import openturns as ot
import math as m
import openturns.testing as ott

ot.PlatformInfo.SetNumericalPrecision(5)
ot.TESTPREAMBLE()
ot.RandomGenerator.SetSeed(0)

def compute_roughness_sampling(distribution, size=500000):
    """
    Sampling method for computing Roughness
    This allows comparing sampling & integrating methods
    """
    dimension = distribution.getDimension()
    uniformNd = ot.ComposedDistribution([ot.Uniform(0, 1) for i in range(dimension)])
    sequence = ot.SobolSequence(dimension)
    experiment = ot.LowDiscrepancyExperiment(sequence, distribution, size, False)
    sample = experiment.generate()
    pdf = distribution.computePDF(sample)
    return pdf.computeMean()[0]


class Quartic(ot.PythonDistribution):
    def __init__(self):
        super(Quartic, self).__init__(1)
        self.c = 15.0 / 16

    def computeCDF(self, x):
        u = x[0]
        if u <= -1:
            p = 0.0
        elif u >= 1:
            p = 1.0
        else:
            p = 0.5 + 15.0 / 16.0 * u \
                - 5.0 / 8.0 * pow(u, 3) \
                + 3.0 / 16.0 * pow(u, 5)
        return p

    def computePDF(self, x):
        u = x[0]
        if u < -1 or u > 1:
            y = 0.0
        else:
            y = self.c * (1 - u**2)**2
        return y

    def getRange(self):
        return ot.Interval(-1.0, 1.0)

# Using some reference values
# See https://en.wikipedia.org/wiki/Kernel_(statistics)#Kernel_functions_in_common_use
# First Normal dist with default ctor
distribution = ot.Normal()
ott.assert_almost_equal(distribution.getRoughness(),
                        0.5 /m.sqrt(m.pi))

# Dimension 2 (Fix https://github.com/openturns/openturns/issues/1485)
# Indep copula : product of integrales
distribution = ot.Normal(2)
ott.assert_almost_equal(distribution.getRoughness(),
                        compute_roughness_sampling(distribution))

# 2D Normal with scale & correlation
# This allows checking that Normal::getRoughness is well implemented
corr = ot.CorrelationMatrix(2)
corr[1, 0] = 0.3
distribution = ot.Normal([0, 0], [1, 2], corr)
ott.assert_almost_equal(distribution.getRoughness(),
                        compute_roughness_sampling(distribution))

distribution = ot.Epanechnikov()
ott.assert_almost_equal(distribution.getRoughness(), 3/5)

distribution = ot.Triangular()
ott.assert_almost_equal(distribution.getRoughness(), 2/3)

distribution = ot.Distribution(Quartic())
ott.assert_almost_equal(distribution.getRoughness(), 5/7)

# Testing Histogram ==> getSingularities
distribution = ot.HistogramFactory().buildAsHistogram(ot.Uniform(0, 1).getSample(100000))
ott.assert_almost_equal(distribution.getRoughness(), 1.0, 5e-4, 1e-5)
# Compute the roughness using width and height
width = distribution.getWidth()
height = distribution.getHeight()
roughness = sum([width[i] * height[i]**2 for i in range(len(height))])
ott.assert_almost_equal(distribution.getRoughness(), roughness)

# Large dimension with independent copula
# With small rho value, we should have results similar to 
# independent copula. But here we use the sampling method
# This allows the validation of this sampling method
corr = ot.CorrelationMatrix(5)
corr[1, 0] = 0.001
distribution = ot.Normal([0]*5, [1]*5, corr)

ott.assert_almost_equal(distribution.getRoughness(),
                        pow(0.5 / m.sqrt(m.pi), 5))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
import openturns as ot
import openturns.testing as ott
import sys


def progress(percent):
    sys.stderr.write('-- progress=' + str(percent) + '%\n')


def stop():
    sys.stderr.write('-- stop?\n')
    return False


# List available algorithms
for algo in ot.Bonmin.GetAlgorithmNames():
    print(algo)

# Definition of objective function
objectiveFunction = ot.SymbolicFunction(
    ['x0', 'x1', 'x2', 'x3'], ['-x0 -x1 -x2'])

# Definition of variables bounds
bounds = ot.Interval([0, 0, 0, 0], [1, 1e308, 1e308, 5], [
                     True, True, True, True], [True, False, False, True])

# Definition of constraints
# Constraints in OpenTURNS are defined as g(x) = 0 and h(x) >= 0
#    No equality constraint -> nothing to do
#    Inequality constraints:
h = ot.SymbolicFunction(['x0', 'x1', 'x2', 'x3'], [
                        '-(x1-1/2)^2 - (x2-1/2)^2 + 1/4', '-x0 + x1', '-x0 - x2 - x3 + 2'])

# Definition of variables types
variablesType = [ot.OptimizationProblemImplementation.BINARY, ot.OptimizationProblemImplementation.CONTINUOUS,
                 ot.OptimizationProblemImplementation.CONTINUOUS, ot.OptimizationProblemImplementation.INTEGER]

# Setting up Bonmin problem
problem = ot.OptimizationProblem(objectiveFunction)
problem.setBounds(bounds)
problem.setVariablesType(variablesType)
problem.setInequalityConstraint(h)

bonminAlgorithm = ot.Bonmin(problem, 'B-BB')
bonminAlgorithm.setStartingPoint([0, 0, 0, 0])
bonminAlgorithm.setMaximumEvaluationNumber(10000)
bonminAlgorithm.setProgressCallback(progress)
bonminAlgorithm.setStopCallback(stop)

ot.ResourceMap.AddAsScalar('Bonmin-bonmin.time_limit', 60)
ot.ResourceMap.AddAsString('Bonmin-mu_oracle', 'loqo')

algos = ot.Bonmin.GetAlgorithmNames()

for algo in algos:
    if algo != "B-iFP":  # Solver B-iFP fails to terminate on this case
        print("MINIMIZATION WITH " + algo)
        bonminAlgorithm.setAlgorithmName(algo)
        bonminAlgorithm.run()
        result = bonminAlgorithm.getResult()
        print(" -- Optimal point = " + result.getOptimalPoint().__str__())
        print(" -- Optimal value = " + result.getOptimalValue().__str__())
        print(" -- Evaluation number = " +
              result.getInputSample().getSize().__str__())
        ott.assert_almost_equal(result.getOptimalPoint(), [
                                1, 1, 0.5, 0], 1, 5e-4)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:

    # Dimension of test
    dimension = 2

    # function
    function = SymbolicFunction("x", "sin(x)")

    # Init of the function
    myFunction = TrendEvaluation(function)
    myFunction.setName("myTrendFunction")

    # We check that for a Point with same values, the function is
    # done by component
    inPoint = Point(dimension, 30.0)

    # result point of the function
    outPoint = myFunction(inPoint)
    print("myFunction=", myFunction)
    print(myFunction.getName(), "( ", inPoint, " ) = ", outPoint)

    # Creation of a Sample
    size = 10
    inSample = Sample(size, dimension)
    for index in range(size):
        inSample[index] = Point(dimension, index + 1)

    # result of the function
    outSample = myFunction(inSample)
    print(myFunction.getName(), "( ", repr(inSample), " ) = ", repr(outSample))


except:
    import sys
    print("t__TrendEvaluation_std.py",
          sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    # Test the initialization method
    RandomGenerator.SetSeed(0)

    # Test the state accessor for reading
    initialState = RandomGenerator.GetState()
    print("initialState=", repr(initialState))
    # Test the generation of one realization
    realization = RandomGenerator.Generate()
    print("one realization=%.6f" % realization)
    state = RandomGenerator.GetState()
    print("state=", repr(state))
    print("new state index=", state.getIndex())

    # Test the generation of several realizations
    size = 10000
    several = RandomGenerator.Generate(size)
    print(size, " realizations, first=%.6f" %
          several[0], ", last=%.6f" % several[size - 1])

    # Test the state accessor for writing
    RandomGenerator.SetState(initialState)
    print("restoring initial state, one realization=%.6f" %
          RandomGenerator.Generate())
    # Test the uniformity of the random generator
    size = 100000
    manyPoints = RandomGenerator.Generate(size)
    slice = 10
    frequencies = Point(slice, 0.0)
    mean = 0.0
    for i in range(size):
        frequencies[int(slice * manyPoints[i])] = frequencies[
            int(slice * manyPoints[i])] + 1
        mean = mean + manyPoints[i]

    mean = mean / size
    print("mean of", size, "realizations= %.12f" % mean)
    for i in range(slice):
        frequencies[i] = frequencies[i] / size
        print("frequency in slice [", (
            1.0 * i) / slice, ",", (i + 1.0) / slice, "]=", frequencies[i])

    print("One integer generation in [[0, ", slice - 1, "]]=",
          RandomGenerator.IntegerGenerate(slice))
    print("Another integer generation in [[0, ", slice - 1, "]]=",
          RandomGenerator.IntegerGenerate(slice))
    manyUnsignedInteger = RandomGenerator.IntegerGenerate(size, slice)
    frequencies = Point(slice, 0.0)
    mean = 0.0
    for i in range(size):
        frequencies[manyUnsignedInteger[i]] = frequencies[
            manyUnsignedInteger[i]] + 1
        mean = mean + manyUnsignedInteger[i]

    mean = mean / size
    print("mean of ", size, " integer realizations=", mean)
    for i in range(slice):
        frequencies[i] = frequencies[i] / size
        print("frequency for value ", i, "=", frequencies[i])

except:
    import sys
    print("t_RandomGenerator.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    laguerre = LaguerreFactory(2.5, LaguerreFactory.PROBABILITY)
    print("laguerre=", laguerre)
    laguerre = LaguerreFactory(2.5, LaguerreFactory.ANALYSIS)
    print("laguerre=", laguerre)
    laguerre = LaguerreFactory(2.5)
    print("laguerre=", laguerre)
    for i in range(10):
        print("laguerre(", i, ")=", laguerre.build(i))
    roots = laguerre.getRoots(10)
    print("laguerre(10) roots=", repr(roots))
    nodes, weights = laguerre.getNodesAndWeights(10)
    print("laguerre(10) nodes=", nodes, "and weights=", weights)
except:
    import sys
    print("t_LaguerreFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])
